<?php

if ( ! class_exists( 'Chilexpress_Woo_Oficial_Shipping_Method' ) ) {
	class Chilexpress_Woo_Oficial_Shipping_Method extends WC_Shipping_Method {
		
		const DESCRIPTION = 'Envios con Chilexpress';
		const ENABLED_KEY = 'enabled';
		const TITLE_KEY = 'tittle';

		/**
		 * Constructor for your shipping class
		 *
		 * @access public
		 * @return void
		 */
		public function __construct() {
			$this->id                 = 'chilexpress_woo_oficial';
			$this->method_title       = 'Chilexpress';
			$this->method_description = self::DESCRIPTION;

			$this->coverage_data = new Chilexpress_Woo_Oficial_Coverage();

			// Availability & Countries
            $this->availability = 'including';
            $this->countries = array(
                'CL' // Chile
            );

			$this->init();

			$this->enabled = isset( $this->settings[self::ENABLED_KEY] ) ? $this->settings[self::ENABLED_KEY] : 'yes';
			$this->title = isset( $this->settings[self::TITLE_KEY] ) ? $this->settings[self::TITLE_KEY] : self::DESCRIPTION;
		}

		/**
		 * Init your settings
		 *
		 * @access public
		 * @return void
		 */
		function init() {
			// Load the settings API
			$this->init_form_fields(); 
			$this->init_settings(); 

			// Save settings in admin if you have any defined
			add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
		}


		/**
		 * Define settings field for this shipping
		 * @return void 
		 */
		function init_form_fields() { 
			$this->form_fields = array(

				'enabled' => array(
					'title' => 'Habilitar', // NOSONAR
					'type' => 'checkbox',
					'description' => 'Habilitar este método de envío.',
					'default' => 'yes'
				),

				'title' => array(
					'title' => 'Title',
					'type' => 'text',
					'description' =>  'Titulo a mostrar en el sitio',
					'default' => self::DESCRIPTION
				)

			);

        }

		/**
		 * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
		 *
		 * @access public
		 * @param mixed $package
		 * @return void
		 */
		public function calculate_shipping( $package = array() ) {
			// We will add the cost, rate and logics in here
			$weight = 0;
			$price = 0;
			$biggest_product = false;
			$biggest_size = 0;
			$options = get_option( 'chilexpress_woo_oficial_general' );

			$i = 0;
            foreach ( $package['contents'] as $item_id => $values ) 
            { 
                $_product = $values['data']; 
                $dimensions = $_product->get_dimensions(false);

                if ($dimensions["width"]!="" && $dimensions["height"]!="" && $dimensions["length"]!="" && "$item_id"=="$item_id"."" ) {
                	if( $biggest_size < $_product->get_height() * $_product->get_width() *$_product->get_length())
                	{
                		$biggest_size = $_product->get_height() * $_product->get_width() * $_product->get_length();
                		$biggest_product = $_product;
                	}
                	$i++;
                }
                else if ($dimensions["width"] == "" && $dimensions["height"] == "" && $dimensions["length"] == "") {
                	if( $biggest_size < floatval($options['alto_producto_defecto']) * floatval($options['ancho_producto_defecto']) * floatval($options['largo_producto_defecto']))
                	{
                		$biggest_size = floatval($options['alto_producto_defecto']) * floatval($options['ancho_producto_defecto']) * floatval($options['largo_producto_defecto']);
                		$biggest_product = $_product;
                		
                		$biggest_product->set_height( floatval($options['alto_producto_defecto']) );
						$biggest_product->set_width( floatval($options['ancho_producto_defecto']) );
						$biggest_product->set_length( floatval($options['largo_producto_defecto']) );
                	}
                	$i++;
                }

                if ($_product->get_weight() != "" && $_product->get_weight() != "0") {
                	$weight = $weight + $_product->get_weight() * $values['quantity']; 
                }
                else{
                	$weight = $weight + floatval($options['peso_producto_defecto']) * $values['quantity'];	
                }

				$price += $_product->get_price() * $values['quantity'];
            }

            $optionsRC = get_option( 'chilexpress_woo_oficial_region_comuna' );
			$api = new Chilexpress_Woo_Oficial_API();
			$region = $package['destination']['state'];
			$comuna = $package['destination']['city'];
			$regionHabilitada = false;
			$comunaHabilitada = false;

			foreach ($optionsRC['regiones_habilitadas'] as $key => $regionH) {
				if($region == $regionH){
					$regionHabilitada = true;
				}	
			}

			foreach ($optionsRC['comunas_habilitadas'] as $key => $comunaH) {
				if($comuna == $comunaH){
					$comunaHabilitada = true;
				}	
			}

			if ($comuna == NULL || $region == NULL) {
				$rate = array(
                    'id' => $this->id,
                    'label' => "Debe seleccionar una Comuna para obtener un estimado.", // NOSONAR
                    'cost' => -1
                );
                
                $this->add_rate( $rate );
				return;
			} 
			else if( $regionHabilitada == false ){
				return false;
				/*$rate = array(
                    'id' => $this->id,
                    'label' => "La region seleccionada no esta disponible.", // NOSONAR
                    'cost' => -1
                );
                
                $this->add_rate( $rate );
				return;*/
			}
			else if( $comunaHabilitada == false){
				return false;
				/*$rate = array(
                    'id' => $this->id,
                    'label' => "La comuna seleccionada no esta disponible.", // NOSONAR
                    'cost' => -1
                );

                $this->add_rate( $rate );
				return;*/
			}
			else {
				if ($biggest_product) {
					$response = $api->obtener_cotizacion($options['comuna_origen'],  $comuna , $weight, 
					$biggest_product->get_height(), 
					$biggest_product->get_width(), 
					$biggest_product->get_length(), $price);
				} else {
					$rate = array(
						'id' => $this->id,
						'label' => "Ningun producto que eligió tiene fijado su tamaño, comuniquese con el administrador",  // NOSONAR
						'cost' => -1
					);
					$this->add_rate( $rate );
					return;
				}
				if ( is_wp_error( $response ) ) {
					$error_message = $response->get_error_message();
					$rate = array(
						'id' => $this->id,
						'label' => "Hubo un error: $error_message", // NOSONAR
						'cost' => -1
					);
					$this->add_rate( $rate );
				} else {

					$lowest_service = NULL;
					foreach($response as $soption)
					{
						//if($soption->serviceTypeCode >= 3)
						//{
							/*if($lowest_service == NULL)
							{
								$lowest_service = $soption;
							}
							else
							{
								if($lowest_service->serviceTypeCode > $soption->serviceTypeCode)
								{
									$lowest_service = $soption;
								}
							}*/

							$rate = array(
								'id'    => $this->id.':'. $soption->serviceTypeCode,
								'label' => 'Chilexpress - '. $soption->serviceDescription,
								'cost'  => $soption->serviceValueDiscount
							);

							$this->add_rate( $rate );
						//}
					}
					
					
				}
			}
		}
	}
}
